@@ -21,6 +21,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#include <config.h>
 #include <pango/pango-types.h>
 #include "mprefixups.h"
 #include <stdio.h>
@@ -74,7 +75,6 @@ void indic_mprefixups_apply(MPreFixups *mprefixups, PangoOTBuffer *buffer)
 	PangoOTGlyph *glyphs;
 	int n_glyphs;
 	PangoOTGlyph *mpreSave;
-	int *clusterSave;
 
 	/* determine post GSUB location of baseIndex and mpreIndex */
 
@@ -98,7 +98,6 @@ void indic_mprefixups_apply(MPreFixups *mprefixups, PangoOTBuffer *buffer)
 	mpreDest   = baseGlyph - mpreCount;
 
 	mpreSave    = g_new (PangoOTGlyph, mpreCount);
-	clusterSave = g_new (int, mpreCount);
 
 	for (i = 0; i < mpreCount; i += 1) {
 	    mpreSave[i] = glyphs[mpreGlyph + i];
