@@ -34,7 +34,11 @@ struct _PangoAnalysis
   PangoEngineShape *shape_engine;
   PangoEngineLang  *lang_engine;
   PangoFont *font;
-  guint8 level;
+
+  guint level : 8;
+  guint gravity : 3; /* PangoGravity */
+  guint centered_baseline : 1; /* gboolean */
+
   PangoLanguage *language;
   GSList *extra_attrs;
 };
@@ -47,6 +51,10 @@ struct _PangoItem
   PangoAnalysis analysis;
 };
 
+#define PANGO_TYPE_ITEM (pango_item_get_type ())
+
+GType pango_item_get_type (void) G_GNUC_CONST;
+
 PangoItem *pango_item_new   (void);
 PangoItem *pango_item_copy  (PangoItem  *item);
 void       pango_item_free  (PangoItem  *item);
