@@ -152,8 +152,7 @@ pango_fc_font_finalize (GObject *object)
   if (priv->decoder)
     _pango_fc_font_set_decoder (fcfont, NULL);
 
-  if (priv->char_to_glyph_cache)
-    g_free (priv->char_to_glyph_cache);
+  g_free (priv->char_to_glyph_cache);
 
   G_OBJECT_CLASS (pango_fc_font_parent_class)->finalize (object);
 }
@@ -263,9 +262,11 @@ pango_fc_font_find_shaper (PangoFont     *font,
 			   guint32        ch)
 {
   PangoMap *shaper_map = NULL;
+  PangoScript script;
 
   shaper_map = pango_fc_get_shaper_map (language);
-  return (PangoEngineShape *)pango_map_get_engine (shaper_map, ch);
+  script = pango_script_for_unichar (ch);
+  return (PangoEngineShape *)pango_map_get_engine (shaper_map, script);
 }
 
 static PangoCoverage *
